// -*- c++ -*-
//------------------------------------------------------------------------------
// $Id: MultichoiceQuestionDB.cpp,v 1.6 2009/07/21 03:07:58 vlg Exp $
//------------------------------------------------------------------------------
//                            MultichoiceQuestionDB.cpp
//------------------------------------------------------------------------------
//  Copyright (c) Vladislav Grinchenko
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version
//  2 of the License, or (at your option) any later version.
//------------------------------------------------------------------------------
//
// Date   : Wed, 28 Jan 2009 17:01:42 -0500
//
//------------------------------------------------------------------------------

#include "Granule.h"
#include "Granule-main.h"
#include "MultichoiceQuestionDB.h"
#include "MultichoiceQuestionUI.h"

#include <libxml/parser.h>
#include <libxml/xmlIO.h>
#include <libxml/xpath.h>
#include <libxml/xpathInternals.h>
#include <libxml/parserInternals.h>

/**-----------------------------------------------------------------------------
 *	Implementation of MultichoiceQuestionDB Model.
 **-----------------------------------------------------------------------------
 */
void
MultichoiceQuestionDB::
clear_answers ()
{
	for (int i = 0; i < MAX_CHOICES; i++) {
		m_is_valid_answer [i] = false;
	}
}

MultichoiceQuestionDB::
MultichoiceQuestionDB ()
	:
	m_enabled (false),
	m_multianswer (false),
	m_width (0)
{
	trace_with_mask ("MultiQDB::MultiQDB",GUITRACE);
	clear_answers ();
}

void
MultichoiceQuestionDB::
set_value (ChoiceTag tag_, const ustring& value_, bool valid_answer_)
{
	m_choice_content [tag_] = value_;
	set_valid_answer (tag_, valid_answer_);

}

/**
 * Logic here is a bit counterintuitive:
 * If the card has only a single valid answer and
 * this is that answer, we clear up previously selected
 * answer (if any), and then mark the valid answer slot.
 */
void
MultichoiceQuestionDB::
set_valid_answer (ChoiceTag tag_, bool valid_answer_)
{
	if (!multianswer () && valid_answer_) {
		clear_answers ();
	}

	m_is_valid_answer [tag_] = valid_answer_;
}

/*==============================================================================
 * Load multi-choice question from XML. This syntax of the entry is:
 *
 * <card multichoice="yes">
 *   <front> ... </front>
 *   <multichoice multianswer="yes">
 *       <choice name="A"> ............... </choice>
 *       <choice name="B" valid="yes"> ... </choice>
 *       <choice name="C"> ............... </choice>
 *       <choice name="D" valid="yes"> ... </choice>
 *   </multichoice>
 *   <back/>      <-- empty, answers BD are implied
 *   <example>
 *       Choices <answer name="b"> and <answer name="d"> are valid.
 *   </example>
 * </card>
 *
 * xPaths:
 *   /deck/card[%d]/multichoice
 *   /deck/card[%d]/multichoice/@multianswer
 *
 *   /deck/card[%d]/multichoice/choice
 *   /deck/card[%d]/multichoice/choice/@name
 *   /deck/card[%d]/multichoice/choice/@valid
 *
 *==============================================================================
 */
void
MultichoiceQuestionDB::
load_from_xml_config (xmlDocPtr xml_doc_,
					  int cards_counter_,
					  std::string& error_msg_)
{
	trace_with_mask ("MultiQDB::load_from_xml_config",GUITRACE);

	char xpath [256];

	xmlXPathContextPtr context = 0;
	xmlXPathObjectPtr  result  = 0;
	xmlNodeSetPtr      nodeset = 0;
	xmlNodePtr         node    = 0;
	xmlChar*           key = 0;

	context = xmlXPathNewContext (xml_doc_);
	if (context == NULL) {
		DL ((ASSA::ASSAERR,"Failed to create xml context.\n"));
		return;
	}

	sprintf (xpath, "/deck/card[%d]/multichoice", cards_counter_);
	DL ((GRAPP,"Evaluating %s\n", xpath));
	
	result = xmlXPathEval (BAD_CAST xpath, context);

	nodeset = result->nodesetval;
	if (xmlXPathNodeSetIsEmpty (nodeset))
	{
		DL ((ASSA::ASSAERR,"Empty nodeset!\n"));
		xmlXPathFreeContext (context);
		xmlXPathFreeObject (result);
		return;
	}

	node = nodeset->nodeTab [0];

	/** Is this a multianswer node?
	 */
	key = xmlGetProp (node, BAD_CAST "multianswer");

	DL ((GRAPP,"Processing <%s> tag\n", node->name));

	if (key) {
		DL ((GRAPP,"with multianswer = %s\n", key));
		if (!strcmp ((const char*) key, "yes")) {
			multianswer (true);
		}
		xmlFree (key);
	}
	else {
		DL ((GRAPP,"with multianswer = no\n"));
	}

	ChoiceTag  tag;
	bool       valid_answer;
	int        valid_count = 0;

	m_width = 0;
	xmlNodePtr cur_node = node->xmlChildrenNode;

	while (cur_node != NULL) 
	{
		if (cur_node->type != XML_ELEMENT_NODE) {
			cur_node = cur_node->next;
			continue;
		}

		/** Process <choice ...> ... </choice> tag
		 */
		if (!xmlStrcmp (cur_node->name, (const xmlChar*) "choice"))
		{
			DL((GRAPP,"Found <%s>[%d] tag\n", cur_node->name, m_width));

			/** Parse <choice name="foobar">
			 */
			tag = 0;
			key = xmlGetProp (cur_node, BAD_CAST "name");

			if (key) {
				if      (!strcmp ((const char*) key, "A")) { tag = 0; }
				else if (!strcmp ((const char*) key, "B")) { tag = 1; }
				else if (!strcmp ((const char*) key, "C")) { tag = 2; }
				else if (!strcmp ((const char*) key, "D")) { tag = 3; }
				else if (!strcmp ((const char*) key, "E")) { tag = 4; }
				else {
					DL ((GRAPP,"Syntax error - "
						 "invalid choice name attribute.\n"));
					xmlFree (key);
					goto done;
				}
				DL ((GRAPP,"name = \"%s\"\n", (const char*) key));
				xmlFree (key);
			}
			else {
				DL ((GRAPP,"<choice> tag has no name!\n"));
			}

			/** Parse <choice valid="yes">
			 */
			valid_answer = false;
			key = xmlGetProp (cur_node, BAD_CAST "valid");
			if (key) {
				if (!strcmp ((const char*) key, "yes")) {
					valid_answer = true;
					valid_count++;
					DL ((GRAPP,"this is a valid answer.\n"));
				}
				xmlFree (key);
			}

			/** Fetch <choice> question text
			 */
			key = xmlNodeListGetString (cur_node->doc,
										cur_node->xmlChildrenNode, 
										1);
			if (key) 
			{
				set_value (tag, (const char*) key, valid_answer);
				DL((GRAPP,"choice[%d] = (%s, %s, %s)\n",
					tag, get_choice_tag (tag), (const char*) key, 
					(valid_answer ? "valid" : "not valid")));
				xmlFree (key);
			}
			else {
				DL ((GRAPP,"<choice> tag is empty?!?!\n"));
			}

			if (tag > m_width) {
				m_width = tag;
			}
		}

		cur_node = cur_node->next;
	}

	DL ((GRAPP, "Found %d choices and %d valid answers\n", 
		 m_width + 1, valid_count));

	enable (true);

  done:
	if (result) {
		xmlXPathFreeObject (result);
	}
	if (context) {
		xmlXPathFreeContext (context);
	}
}
	
void
MultichoiceQuestionDB::
save_to_xml_config (xmlTextWriterPtr writer_)
{
	trace_with_mask ("MultiQDB::save_from_xml_config",GUITRACE);


	DL ((GRAPP, "card has %d questions and %s multianswer\n",
		 m_width + 1, (m_multianswer ? "is" : "is not")));
	int ret = xmlTextWriterStartElement (writer_, BAD_CAST "multichoice");

	if (multianswer ()) {
		ret = xmlTextWriterWriteAttribute (writer_, BAD_CAST "multianswer", 
										   BAD_CAST "yes");
	}

	for (int i = 0; i <= m_width; i++) 
	{
		DL ((GRAPP,"Write choice entry %d\n", i));
		write_choice_xml_entry (i, writer_);
	}

	ret = xmlTextWriterEndElement (writer_);
}

/** Update View with data from the Model
 */
void
MultichoiceQuestionDB::
write_choice_xml_entry (int index_, xmlTextWriterPtr writer_)
{
	trace_with_mask ("MultiQDB::write_choice_xml_entry",GUITRACE);
	int ret;

	ret = xmlTextWriterStartElement   (writer_, BAD_CAST "choice");
	ret = xmlTextWriterWriteAttribute (writer_,	BAD_CAST "name", 
									   BAD_CAST get_choice_tag (index_));
	if (m_is_valid_answer [index_]) {
		ret = xmlTextWriterWriteAttribute (writer_, 
										   BAD_CAST "valid", 
										   BAD_CAST "yes");
	}

	ret = xmlTextWriterWriteString (writer_, 
									BAD_CAST m_choice_content [index_].c_str());
	ret = xmlTextWriterEndElement (writer_);
}

const char*
MultichoiceQuestionDB::
get_choice_tag (ChoiceTag tag_)
{
	switch (tag_) {
	    case 0: return "A";
		case 1: return "B";
		case 2: return "C";
		case 3: return "D";
		case 4: return "E";
	}
	return "A";
}

/** Update View with data from the Model
 */
void
MultichoiceQuestionDB::
update_view (MultichoiceQuestionUI*& view_)
{
	for (int tag_idx = 0; tag_idx < MAX_CHOICES; tag_idx++) {
		view_->set_text (tag_idx, m_choice_content [tag_idx]);
	}
}

/** Store data from View to the Model.
 */
/*
void
MultichoiceQuestionDB::
fetch_from_view (ColorsSelectorUI*& view_)
{
	set_text_fg_color   (view_->get_text (TXTFGCLR));
	set_text_bg_color   (view_->get_text (TXTBGCLR));
	set_separator_color (view_->get_text (SEPCLR));
	set_frame_color     (view_->get_text (FRMCLR));
}
*/

  
