// -*- c++ -*-
//------------------------------------------------------------------------------
//                           MultichoiceQuestionDB.h
//------------------------------------------------------------------------------
// $Id: MultichoiceQuestionDB.h,v 1.5 2009/07/21 03:07:58 vlg Exp $
//
//  Copyright (c) Vladislav Grinchenko
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version
//  2 of the License, or (at your option) any later version.
//
// Date: Wed, 28 Jan 2009 16:15:53 -0500
//------------------------------------------------------------------------------
#ifndef MULTICHOICE_QUESTION_DB_H
#define MULTICHOICE_QUESTION_DB_H

#include <inttypes.h>

/** @file MultichoiceQuestionDB.h
 *
 *  An encapsulation of the multichoice answer capability of a Card.
 */

#include "Granule-main.h"

#include <string>
using std::string;

#include <libxml/tree.h>		// xmlDocPtr
#include <libxml/xmlwriter.h>	// xmlTextWriterPtr

#include <glibmm/ustring.h>
using Glib::ustring;

class MultichoiceQuestionUI;

/**
 * MultichoiceQuestionDB holds all possible answers.
 * Single valid answer as well as multiple valid answers are supported.
 * This class represents persisten storage in XML file.
 */

class MultichoiceQuestionDB
{
public:
	/** Maximum allowable number of answers.
	 */
	static const int MAX_CHOICES = 5;

	/** The tag of the choice. The choices are randomly shuffled
	 *  before presented to the user. The ChoiceTag, however, is
	 *  used internally for text substitution and validation of the
	 *  answer(s) provided by the user.
	 */
	typedef uint32_t ChoiceTag;

public:
	MultichoiceQuestionDB ();

	/// Is this object enabled?
	bool is_enabled () const { return m_enabled; }

	/// Configure multi-answer option
	void multianswer (bool v_) { m_multianswer = v_; }

	/// Is this a multi-answer question?
	bool multianswer () const { return m_multianswer; }

	/** Set choice value
	 *  @tag_   The answer tag
	 *  @value_ The text of the choice statement
	 *  @answer Mark this choice as valid if true. If multi-answer is disabled,
	 *  this becomes the only valid answer.
	 */
	void set_value (ChoiceTag tag_,
					const ustring& value_,
					bool valid_answer_ = false);

	/** Update the validity state of the choice.
	 */
	void set_valid_answer (ChoiceTag tag_, bool valid_answer_ = true);

	/** Retrieve the choice. The caller should check if the returned
	 *  value is not empty.
	 */
	ustring get_value (ChoiceTag tag_) { return m_choice_content [tag_]; }

	/** Is this the answer?
	 */
	bool is_valid_answer (ChoiceTag tag_) { return m_is_valid_answer [tag_]; }
	
	/** Persisten storage with XML (Deck) file.
	 */
    void load_from_xml_config (xmlDocPtr parser_,
							   int cards_counter_,
							   string& error_msg_);

    void save_to_xml_config (xmlTextWriterPtr writer);

	/// Update View with data from Model
	void update_view (MultichoiceQuestionUI*& view_);

	/// Store View values in Model
//	void fetch_from_view (MultichoiceTag tag_, MultichoiceQuestionUI*& view_);

public:
    /// Convert index to alphabetical letter
	static const char* get_choice_tag (ChoiceTag tag_);

private:
	/// Enable multi-choice capability
	void enable (bool v_) { m_enabled = v_; }

	/// Mark all answers as false
	void clear_answers ();

	/// Write set of choices for one card
	void write_choice_xml_entry (int index_, xmlTextWriterPtr writer_);

private:
	/// Is multi-choice enabled?
	bool m_enabled;

	/// Is it a multi-answer question
	bool m_multianswer;

	/// All choices
	ustring m_choice_content [MAX_CHOICES];

	/// At least one valid answer
	bool m_is_valid_answer [MAX_CHOICES];

	/// Number of offered answers for the question: [0; MAX_CHOICES]
	int m_width;
};

//------------------------------------------------------------------------------
// Implementation
//------------------------------------------------------------------------------


#endif /* MULTICHOICE_QUESTION_DB_H */
